#include <signal.h>
#include <io.h>
#include "types.h"
#include "hardware.h"
#include "global.h"
#include "buttons.h"

#define BUTTON_DEBOUNCE_SCANS 3

extern volatile uint16 iflags;

void read_buttons(void)
{
    static uint8 debounce_cnt = 0;
    static uint8 curr_state = 0, prev_state = 0;
    uint8 new_state = 0;

    new_state = BUTTON_PORT & BUTTON_MASK;

    if (new_state != curr_state)
    {
        if (new_state == prev_state)
        {
            debounce_cnt++;
        }
        else
        {
            debounce_cnt = 0;
        }

        prev_state = new_state;

        if (debounce_cnt == BUTTON_DEBOUNCE_SCANS)
        {
            // Button

            if ((curr_state & BUTTON_FIRE) && ((new_state & BUTTON_FIRE) == 0))
            {
                iflags |= IF_BTN_FIRE_PRESS;
            }

            if ((curr_state & BUTTON_LEFT) && ((new_state & BUTTON_LEFT) == 0))
            {
                iflags |= IF_BTN_LEFT_PRESS;
            }

            if ((curr_state & BUTTON_RIGHT) && ((new_state & BUTTON_RIGHT) == 0))
            {
                iflags |= IF_BTN_RIGHT_PRESS;
            }

            curr_state = new_state;
            debounce_cnt = 0;
        }
    }
    else
    {
        debounce_cnt = 0;
    }
}
