#include <signal.h>
#include <io.h>
#include "types.h"
#include "hardware.h"
#include "global.h"
#include "display.h"
#include "timer.h"

typedef enum {CS_BRIGHT, CS_DIM} CURSORSTATE_T;

uint8 u8DisplayBuf[4];
uint16 u16CursorPos = 3;
CURSORSTATE_T csCursorState = CS_DIM;

#define INT_BRIGHT  (2500)
#define INT_DIM     (750)

void set_cursorpos(uint16 u16pos)
{
    u16CursorPos = u16pos;
}

uint16 u16get_cursorpos(void)
{
    return u16CursorPos;
}

void cursor_left(void)
{
    if (u16CursorPos > 0)
    {
        u16CursorPos--;
    }
}

void cursor_right(void)
{
    if (u16CursorPos < 3)
    {
        u16CursorPos++;
    }
}

void cursor_blink(void)
{
    if (csCursorState == CS_DIM)
    {
        csCursorState = CS_BRIGHT;
    }
    else
    {
        csCursorState = CS_DIM;
    }
}

void display_refresh(void)
{
    static uint16 u16CurrentDigit = 0;

    // Turn off all digits and segments
    DIGIT_PORT &= ~DIGIT_MASK;
    SEG_PORT = 0x00;

    // Determine next digit to lit
    u16CurrentDigit++;
    if (u16CurrentDigit > 3)
    {
        u16CurrentDigit = 0;
    }

    // set cursor intensity
    if (u16CurrentDigit == u16CursorPos)
    {
        if (csCursorState == CS_BRIGHT)
        {
            set_dpwm(INT_BRIGHT);
        }
        else
        {
            set_dpwm(INT_DIM);
        }
    }
    else
    {
        set_dpwm(INT_BRIGHT);
    }

    // let there be light
    switch(u16CurrentDigit)
    {
        case 0:
            SEG_PORT = u8decode7seg(u8DisplayBuf[0]);
            DIGIT_PORT |= DIGIT_DIGIT1000;
            break;

        case 1:
            SEG_PORT = u8decode7seg(u8DisplayBuf[1]);
            DIGIT_PORT |= DIGIT_DIGIT100;
            break;

        case 2:
            SEG_PORT = u8decode7seg(u8DisplayBuf[2]);
            DIGIT_PORT |= DIGIT_DIGIT10;
            break;

        case 3:
            SEG_PORT = u8decode7seg(u8DisplayBuf[3]);
            DIGIT_PORT |= DIGIT_DIGIT1;
            break;

        default:
            SEG_PORT = 0x00;
            DIGIT_PORT = 0x00;
    }
}

uint8 u8decode7seg(uint8 u8Digit)
{
    uint8 u8Tmp;

    const static uint8 u8Segments[16] =
    {
    DIGIT_0, DIGIT_1, DIGIT_2, DIGIT_3,
    DIGIT_4, DIGIT_5, DIGIT_6, DIGIT_7,
    DIGIT_8, DIGIT_9, DIGIT_MINUS, 0,
    0,0,0,0
    };

    u8Tmp = u8Segments[u8Digit & 0x0F];

    if (u8Digit & 0x80)
    {
        u8Tmp |= SEG_DP;
    }

    return u8Tmp;
}
