#include <signal.h>
#include <io.h>
#include "types.h"
#include "hardware.h"
#include "global.h"
#include "encoder.h"

#define ENCODER_DEBOUNCE_SCANS 3

extern volatile uint16 iflags;

void read_encoder(void)
{
  static uint8 enc_debounce_cnt = 0;
  static uint8 curr_encstate = 0, prev_encstate = 0;
  uint8 new_encstate = 0, s0, s1;
  static int16 i16SubTicks = 0;

  new_encstate = ENCODER_PORT & (ENCODER_MASK | ENCODER_BUTTON);

  if (new_encstate != curr_encstate)
  {
    if (new_encstate == prev_encstate)
    {
        enc_debounce_cnt++;
    } else {
        enc_debounce_cnt = 0;
    }

    prev_encstate = new_encstate;

    if (enc_debounce_cnt == ENCODER_DEBOUNCE_SCANS)
    {

            s0 = curr_encstate & ENCODER_MASK;
            s1 = new_encstate & ENCODER_MASK;

        // Rotation

        if ((s0 == ENCODER_STATE_0) && (s1 == ENCODER_STATE_1))
        {
                i16SubTicks++;
        }
        else if ((s0 == ENCODER_STATE_1) && (s1 == ENCODER_STATE_2))
        {
                i16SubTicks++;
        }
        else if ((s0 == ENCODER_STATE_2) && (s1 == ENCODER_STATE_3))
        {
                i16SubTicks++;
        }
        else if ((s0 == ENCODER_STATE_3) && (s1 == ENCODER_STATE_0))
        {
                i16SubTicks++;
        }
        else if ((s0 == ENCODER_STATE_0) && (s1 == ENCODER_STATE_3))
        {
                i16SubTicks--;
        }
        else if ((s0 == ENCODER_STATE_3) && (s1 == ENCODER_STATE_2))
        {
                i16SubTicks--;
        }
        else if ((s0 == ENCODER_STATE_2) && (s1 == ENCODER_STATE_1))
        {
                i16SubTicks--;
        }
        else if ((s0 == ENCODER_STATE_1) && (s1 == ENCODER_STATE_0))
        {
                i16SubTicks--;
        }

        if (i16SubTicks > 3)
        {
            i16SubTicks = 0;
            iflags |= IF_ENC_UP;
        }
        else if (i16SubTicks < -3)
        {
            i16SubTicks = 0;
            iflags |= IF_ENC_DN;
        }

        // Button

        if (((curr_encstate & ENCODER_BUTTON) == ENCODER_BUTTON) && ((new_encstate & ENCODER_BUTTON) == 0))
        {
            iflags |= IF_ENC_BUTTON_DOWN;
        }

        curr_encstate = new_encstate;
        enc_debounce_cnt = 0;
    }
  }
  else
  {
    enc_debounce_cnt = 0;
  }
}
