/*
**
** Inductor tester
**
** (c) JTA 2010
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <io.h>
#include <stdio.h>
#include <string.h>
#include "types.h"
#include "global.h"
#include "init.h"
#include "hardware.h"
#include "timer.h"
#include "timerb.h"
#include "udelay.h"
#include "display.h"

#define MAX_PULSEWIDTH (9999)
#define MIN_PULSEWIDTH (1)
#define PULSEWIDTH_1US (1)

volatile uint16 iflags = 0;
volatile uint16 u16PulseWidth = 0;

extern uint8 u8DisplayBuf[];
extern volatile uint16 u16swtimer;

void ShowPulseLength(uint16 u16time);
void RightJustify(char *psString, uint16 u16width);
void inc_pulsewidth(void);
void dec_pulsewidth(void);

/*
** Main function
*/

int main(void)
{
    wdt_init();
    io_init();
    osc_init();
    init_timer1();
    init_timerb();

    eint();

    // Main Loop

    u8DisplayBuf[0] = 0xA;
    u8DisplayBuf[1] = 0xA;
    u8DisplayBuf[2] = 0xA;
    u8DisplayBuf[3] = 0xA;

    u16PulseWidth = 1*PULSEWIDTH_1US;
    ShowPulseLength(u16PulseWidth);

    while(1)
    {
        if (iflags & IF_ENC_UP)
        {
            inc_pulsewidth();
            ShowPulseLength(u16PulseWidth);
            iflags &= ~IF_ENC_UP;
        }

        if (iflags & IF_ENC_DN)
        {
            dec_pulsewidth();
            ShowPulseLength(u16PulseWidth);
            iflags &= ~IF_ENC_DN;
        }

        if (iflags & IF_DISP_FLASH)
        {
            cursor_blink();
            iflags &= ~IF_DISP_FLASH;
        }

        if (iflags & IF_ENC_BUTTON_DOWN)
        {
            iflags &= ~IF_ENC_BUTTON_DOWN;
        }

        if (iflags & IF_BTN_LEFT_PRESS)
        {
            cursor_right();
            ShowPulseLength(u16PulseWidth);            
            iflags &= ~IF_BTN_LEFT_PRESS;
        }

        if (iflags & IF_BTN_RIGHT_PRESS)
        {
            cursor_left();
            ShowPulseLength(u16PulseWidth);
            iflags &= ~IF_BTN_RIGHT_PRESS;
        }

        if (iflags & IF_BTN_FIRE_PRESS)
        {
            SetPulseWidth(u16PulseWidth);
            iflags &= ~IF_BTN_FIRE_PRESS;
        }
    }

    return 0;
}

uint16 u16CursorPosToIncrement(uint16 u16CursorPos)
{
    uint16 u16Inc;
    
    switch (u16CursorPos)
    {
        case 0:
            u16Inc = 1000;
            break;
            
        case 1:
            u16Inc = 100;
            break;
            
        case 2:
            u16Inc = 10;
            break;
            
        case 3:
            u16Inc = 1;
            break;
            
        default:
            u16Inc = 1;
    }
    
    return u16Inc;
}

void inc_pulsewidth(void)
{
    uint16 u16CursorPos;
    uint16 u16Inc;
    
    u16CursorPos = u16get_cursorpos();
    u16Inc = u16CursorPosToIncrement(u16CursorPos);
    
    if ((u16PulseWidth + u16Inc) > MAX_PULSEWIDTH)
    {
        u16Inc = MAX_PULSEWIDTH - u16PulseWidth;
    }
    
    u16PulseWidth += u16Inc;
}

void dec_pulsewidth(void)
{
    uint16 u16CursorPos;
    uint16 u16Inc;
    
    u16CursorPos = u16get_cursorpos();
    u16Inc = u16CursorPosToIncrement(u16CursorPos);
    
    if (u16PulseWidth < (u16Inc + MIN_PULSEWIDTH))
    {
        u16Inc = u16PulseWidth - MIN_PULSEWIDTH;
    }
    
    u16PulseWidth -= u16Inc;
}

#define TIMESTR_WIDTH 4

void ShowPulseLength(uint16 u16PulseLength)
{
    char u8OutBuf[TIMESTR_WIDTH+1];
    uint16 u16i, u16j = 0;
    bool bLeadingBlank = TRUE;
    uint16 u16CursorPos = u16get_cursorpos();

    utoa(u16PulseLength, u8OutBuf, 10);
    RightJustify(u8OutBuf, TIMESTR_WIDTH);

    for (u16i=0;u16i<TIMESTR_WIDTH;u16i++)
    {
        if (u16i >= u16CursorPos)
        {
            bLeadingBlank = FALSE;
        }
        
        switch(u8OutBuf[u16i])
        {
            case '.':
                break;
            case '-':
                u8DisplayBuf[u16j++] = 0xA;
                bLeadingBlank = FALSE;
                break;
            case '0':
                if (bLeadingBlank) 
                {
                    u8DisplayBuf[u16j++] = 0xF;
                }
                else
                {
                    u8DisplayBuf[u16j++] = 0x0;
                }
                break;
            case 0x20:
                u8DisplayBuf[u16j++] = 0xF;
                break;
            default:
                u8DisplayBuf[u16j++] = u8OutBuf[u16i] - '0';
                bLeadingBlank = FALSE;
        }
    }
}

void RightJustify(char *psString, uint16 u16width)
{
    uint16 u16len;
    uint16 u16i, u16j, u16paddingwidth;

    u16len = strlen(psString);

    u16j = u16width-1;

    for(u16i=u16len;u16i>0;u16i--)
    {
        psString[u16j--] = psString[u16i-1];
    }

    u16paddingwidth = u16width-u16len;

    for(u16i=0;u16i<u16paddingwidth;u16i++)
    {
        psString[u16i] = '0';//0x20;
    }
    psString[u16width] = '\0';
}
