#include <signal.h>
#include <io.h>
#include "types.h"
#include "hardware.h"
#include "global.h"
#include "encoder.h"
#include "buttons.h"
#include "display.h"
#include "timer.h"

extern volatile uint16 iflags;
volatile uint16 u16swtimer = 0;

#define DISPLAY_MULTIPLEX_DIVIDER 10
#define MILLISEC_DIVIDER 5
#define DISPLAY_FLASH_DIVIDER 1000

void init_timer1(void)
{
  TACCR0 = 2499; // 4000 Hz interrupt frequency
  TACCTL0 = CM_0 | CCIS_0 | OUTMOD_0;

  // display PWM
  TACCR1 = 0;
  TACCTL1 = CM_0 | CCIS_0 | SCS | CCIS_0 | CLLD_1 | OUTMOD_7;

  TACTL = TASSEL_SMCLK | ID_DIV1 | MC_UPTO_CCR0 | TACLR | TAIE;
}

void set_dpwm(uint16 u16pwm)
{
    TACCR1 = u16pwm;
}

interrupt(TIMERA1_VECTOR) TimerInt(void)
{
    static uint16 u16DisplayUpdateCounter = DISPLAY_MULTIPLEX_DIVIDER;
    static uint16 u16msDividerCnt = MILLISEC_DIVIDER;
    static uint16 u16DisplayFlashDividerCnt = DISPLAY_FLASH_DIVIDER;

    switch(TAIV)
    {
        case TAIV_OVERFLOW:
            read_encoder();
            read_buttons();

            u16DisplayUpdateCounter--;
            if (u16DisplayUpdateCounter == 0)
            {
                u16DisplayUpdateCounter = DISPLAY_MULTIPLEX_DIVIDER;
                display_refresh();
            }
            u16msDividerCnt--;
            if (u16msDividerCnt == 0)
            {
                if (u16swtimer > 0)
                {
                    u16swtimer--;
                }

                u16msDividerCnt = MILLISEC_DIVIDER;
            }

            if (u16DisplayFlashDividerCnt > 0)
            {
                u16DisplayFlashDividerCnt--;
            }
            else
            {
                iflags |= IF_DISP_FLASH;
                u16DisplayFlashDividerCnt = DISPLAY_FLASH_DIVIDER;
            }
        break;

        default:
            nop();
    }
}
